@@ -262,6 +262,9 @@ class JoinComputation(Computation):
             for i, stream in enumerate(self.input_streams):
                 if not stream.empty():
                     item = stream.get()
+                    if isinstance(item.value, DelayedException):
+                        self.put(item)
+                        return
                     for args in manager.new_function_arguments_for(i, item):
                         Engine.run_async(self.f, args, callback=self.put)
                         started_computation = True
@@ -311,9 +314,6 @@ class ZipComputation(Computation):
         # sizes has size 0 means: we only had inputs of size 1
         return 1
 
-    def _setup_computations(self):
-        cip = self._collect_data_from_constant_input_streams()
-
     def _collect_data_from_constant_input_streams(self):
         constant_inputs = [None] * len(self.input_streams)
         for i, stream in enumerate(self.input_streams):
@@ -344,6 +344,11 @@ class ZipComputation(Computation):
         n = 0
         # wait and get the constant args which we need anyway:
         cip = self._collect_data_from_constant_input_streams()
+        for item in cip:
+            if item is not None:
+                if isinstance(item.value, DelayedException):
+                    self.put(item)
+                    return
         collected_input_data = dict()
         # only constant inputs ?
         if self.size() == 1:
@@ -353,6 +358,10 @@ class ZipComputation(Computation):
                 avail = self._check_for_any_new_data()
                 if avail:
                     for args in self._assemble_function_arguments(collected_input_data, cip):
+                        for arg in args:
+                            if isinstance(arg, DelayedException):
+                                self.put(arg)
+                                return
                         Engine.run_async(self.f, args, callback=self.put)
                         n += 1
                 else:
